@@ -1,5 +1,5 @@
 from .affinity import ea, x2p
-from .error import error_ee, error_ee_cpu, error_ee_cuda
+from .error import error_ee, error_ee_split
 from .linearsearch import ls_ee
 from .visualize import scatter, scatter_with_colorbar
 from .embeddingsLoss import eloss
@@ -7,8 +7,7 @@ from .embeddingsLoss import eloss
 __all__ = ['ea',
            'x2p',
            'error_ee',
-           'error_ee_cpu',
-           'error_ee_cuda',
+           'error_ee_split',
            'ls_ee',
            'scatter',
            'scatter_with_colorbar',
